import typing as T
import edge_orm
from fastapi import Request
from functools import cached_property
from edgedb import AsyncIOClient
from pydantic import BaseModel, Field, EmailStr
from quilt import DisplayException, logs
from quilt.vendors.Firebase import auth
from edge_orm import UNSET, UnsetType

logger = logs.create_logger(__name__)


class Identities(BaseModel):
    # apple.com -> array of apple ids
    apple_com: list[str] | None = Field(None, alias="apple.com")
    email: list[str] | None = None


class Firebase(BaseModel):
    sign_in_provider: str  # apple.com for apple or password for email
    identities: Identities


class InvalidAuthHeader(DisplayException):
    pass


class AuthExpirationException(DisplayException):
    pass


class AuthTokenPayload(BaseModel):
    name: str | None = None  # for apple this will exist, for email this will not
    iss: str
    aud: str
    auth_time: int
    user_id: str
    sub: str
    iat: int
    exp: int
    email: EmailStr | None = None
    email_verified: bool | None = None
    # firebase: Firebase # not used now so don't want to spend time parsing
    phone_number: str | None = None

    @classmethod
    def from_token_firebase(cls, bearer_token: str | bytes) -> "AuthTokenPayload":
        try:
            raw_jwt = auth.verify_id_token(bearer_token)
        except auth.InvalidIdTokenError as e:
            logger.debug(f"{e=}")
            if "Token expired" in str(e):
                raise AuthExpirationException(str(e))
            raise InvalidAuthHeader(str(e))
        return cls(**raw_jwt)


class UserManagerException(Exception):
    pass


UserType = T.TypeVar("UserType", bound=edge_orm.Node)
UserResolverType = T.TypeVar("UserResolverType", bound=edge_orm.Resolver)


class UserMixin(T.Generic[UserType, UserResolverType]):
    def __init__(
        self,
        default_resolver_function: T.Callable[[], UserResolverType],
        default_include_properties_function: T.Callable[[UserResolverType], None],
    ):
        self.default_resolver_function = default_resolver_function
        self.default_include_properties_function = default_include_properties_function

    request: Request
    _user: UserType | None | UnsetType

    @property
    def auth_header(self) -> str | None:
        return self.request.headers.get("Authorization")

    @cached_property
    def jwt_user(self) -> AuthTokenPayload | None:
        auth_header = self.auth_header
        if not auth_header:
            return None
        try:
            [auth_kind, token] = auth_header.split(" ")
        except ValueError:
            logger.error(f"invalid auth header: {auth_header}END")
            raise InvalidAuthHeader("Invalid Authorization Header", traces_rate=0.1)
        return AuthTokenPayload.from_token_firebase(token)

    @property
    def auth_id(self) -> str | None:
        if not self.jwt_user:
            return None
        return self.jwt_user.user_id

    @property
    def admin_auth_id(self) -> str | None:
        # FUTURE when doing admin on behalf of...
        return None

    async def get(
        self,
        given_resolver: T.Optional[UserResolverType] = None,
        *,
        refresh: T.Optional[bool] = False,
        include_properties: T.Optional[bool] = True,
        include_permissions: T.Optional[bool] = True,
        gerror: T.Optional[bool] = True,
        client: T.Optional[AsyncIOClient] = None,
    ) -> UserType | None:
        if refresh:
            self._user = UNSET
        if self._user is not UNSET:
            return self._user
        if not self.auth_id:
            return None
        resolver = given_resolver or self.default_resolver_function()
        if include_properties:
            self.default_include_properties_function(resolver)
        if include_permissions:
            resolver.include_fields("permissions")
        func = resolver.gerror if gerror else resolver.get
        try:
            self._user = await func(auth_id=self.auth_id, client=client)
        except edge_orm.ResolverException:
            raise UserManagerException("Please create an account to access.")
        return self._user

    def set(self, user: UserType | None) -> None:
        self._user = user



import os
import json
import httpx
from quilt.vendors.Firebase import auth
from quilt import logs

logger = logs.create_logger(__name__)


def id_token_from_custom_token(custom_token: bytes) -> bytes:
    logger.debug("getting custom token from jwt_creator_url")
    res = httpx.post(
        os.environ["JWT_CREATOR_URL"],
        json={
            "customToken": custom_token.decode(),
            "firebaseConfig": json.loads(os.environ["FIREBASE_FRONTEND_CONFIG"]),
        },
        timeout=10,
    )
    return res.content


def get_user(phone_number: str) -> auth.UserRecord:
    return auth.get_user_by_phone_number(phone_number)


def bearer_token_from_auth_id(auth_id: str) -> bytes:
    custom_token = auth.create_custom_token(uid=auth_id)
    return id_token_from_custom_token(custom_token)


def bearer_token_from_phone_number(phone_number: str) -> bytes:
    try:
        auth_user = auth.get_user_by_phone_number(phone_number)
        logger.debug(f"got existing auth user with phone_number={phone_number}")
    except auth.UserNotFoundError:
        auth_user = auth.create_user(phone_number=phone_number)
        logger.debug(f"created auth user with phone_number={phone_number}")
    return bearer_token_from_auth_id(auth_user.uid)



import typing as T
from pydantic import BaseModel
from edge_orm import Resolver
from dataclasses import asdict

if T.TYPE_CHECKING:
    from .default_filters import (
        AddToResolverMixin,
        AddToResolverMixinOrder,
    )

ThisResolverType = T.TypeVar("ThisResolverType", bound=Resolver)


class FilterMixin:
    def filter_obj(
        self: ThisResolverType,
        filter: T.Union["AddToResolverMixin", BaseModel, dict[str, T.Any]],
    ) -> ThisResolverType:
        from . import logic
        from app.dbs.edgedb.gen import filters

        if not filter:
            return self
        if not isinstance(filter, BaseModel):
            if not isinstance(filter, dict):
                filter = asdict(filter)
            filter = filters.FilterOrderByName["Filter" + self.model_name](**filter)

        filter_str, variables = logic.node_filter_to_str(f=filter, root_field_name="")
        return self.filter(filter_str, variables)

    def order_obj(
        self: ThisResolverType,
        order: T.Union["AddToResolverMixinOrder", BaseModel, dict[str, T.Any]],
    ) -> ThisResolverType:
        from . import logic
        from app.dbs.edgedb.gen import filters

        if not order:
            return self
        if not isinstance(order, BaseModel):
            if not isinstance(order, dict):
                order = asdict(order)
            order = filters.FilterOrderByName["Order" + self.model_name](**order)

        order_str = logic.order_to_str(order)
        return self.order_by(order_str)



import typing as T
import hashlib
from inspect import isawaitable
from typing import Optional

from sentry_sdk import configure_scope, start_span, set_user

from strawberry.extensions import SchemaExtension
from strawberry.extensions.tracing.utils import should_skip_tracing
from strawberry.types.execution import ExecutionContext
from strawberry.utils.cached_property import cached_property

from devtools import pformat

from quilt import logs, helpers

logger = logs.create_logger(__name__)

class SentryTracingExtension(SchemaExtension):
    def __init__(
        self,
        *,
        execution_context: Optional[ExecutionContext] = None,
    ):
        if execution_context:
            self.execution_context = execution_context

    @cached_property
    def _resource_name(self) -> str:
        assert self.execution_context.query

        query_hash = self.hash_query(self.execution_context.query)

        if self.execution_context.operation_name:
            return f"{self.execution_context.operation_name}:{query_hash}"

        return query_hash

    def hash_query(self, query: str) -> str:
        return hashlib.md5(query.encode("utf-8")).hexdigest()

    def on_request_start(self) -> None:
        self._operation_name = self.execution_context.operation_name or helpers.operation_name_from_query(self.execution_context.query)
        context = self.execution_context.context
        name = self._operation_name or 'Anonymous Query'
        if jwt_user := context.auth.jwt_user:
            d = {
                "operation": self._operation_name,
                "name": jwt_user.name,
                "auth_id": jwt_user.user_id,
                "request_id": str(context.request_id),
            }
            logger.info(pformat(d))
            set_user(
                {
                    "id": jwt_user.user_id,
                    "username": jwt_user.name,
                    "email": jwt_user.email,
                    "ip_address": context.req.ip_address,
                }
            )

        with configure_scope() as scope:
            if scope.span:
                self.gql_span = scope.span.start_child(
                    op="gql",
                    description=name,
                )
            else:
                self.gql_span = start_span(
                    op="gql",
                )
            if scope.transaction:
                scope.transaction.name = name
        operation_type = "query"

        assert self.execution_context.query

        if self.execution_context.query.strip().startswith("mutation"):
            operation_type = "mutation"
        if self.execution_context.query.strip().startswith("subscription"):
            operation_type = "subscription"

        self.gql_span.set_tag("graphql.operation_type", operation_type)
        self.gql_span.set_tag("graphql.resource_name", self._resource_name)
        self.gql_span.set_data("graphql.query", self.execution_context.query)

    def on_request_end(self) -> None:
        self.gql_span.finish()

    def on_validation_start(self) -> None:
        self.validation_span = self.gql_span.start_child(
            op="validation", description="Validation"
        )

    def on_validation_end(self) -> None:
        self.validation_span.finish()

    def on_parsing_start(self) -> None:
        self.parsing_span = self.gql_span.start_child(
            op="parsing", description="Parsing"
        )

    def on_parsing_end(self) -> None:
        self.parsing_span.finish()

    async def resolve(self, _next, root, info, *args, **kwargs) -> T.Any:  # type: ignore
        if should_skip_tracing(_next, info):
            result = _next(root, info, *args, **kwargs)

            if isawaitable(result):  # pragma: no cover
                result = await result

            return result

        field_path = f"{info.parent_type}.{info.field_name}"

        with self.gql_span.start_child(
            op="resolve", description=f"Resolving: {field_path}"
        ) as span:
            span.set_tag("graphql.field_name", info.field_name)
            span.set_tag("graphql.parent_type", info.parent_type.name)
            span.set_tag("graphql.field_path", field_path)
            span.set_tag("graphql.path", ".".join(map(str, info.path.as_list())))

            result = _next(root, info, *args, **kwargs)

            if isawaitable(result):
                result = await result

            return result


class SentryTracingExtensionSync(SentryTracingExtension):
    def resolve(self, _next, root, info, *args, **kwargs) -> T.Any:  # type: ignore
        if should_skip_tracing(_next, info):
            return _next(root, info, *args, **kwargs)

        field_path = f"{info.parent_type}.{info.field_name}"

        with self.gql_span.start_child(
            op="resolve", description=f"Resolving: {field_path}"
        ) as span:
            span.set_tag("graphql.field_name", info.field_name)
            span.set_tag("graphql.parent_type", info.parent_type.name)
            span.set_tag("graphql.field_path", field_path)
            span.set_tag("graphql.path", ".".join(map(str, info.path.as_list())))

            return _next(root, info, *args, **kwargs)



import random
import sentry_sdk
import typing as T
import functools
from inspect import isawaitable

P = T.ParamSpec("P")


def sample_rate(
    rate: float,
) -> T.Callable[[T.Callable[P, T.Any]], T.Callable[P, T.Any]]:
    def outer_wrapper(func: T.Callable[P, T.Any]) -> T.Callable[P, T.Any]:
        @functools.wraps(func)
        async def wrapper(*args: P.args, **kwargs: P.kwargs) -> T.Any:
            with sentry_sdk.configure_scope() as scope:
                scope.transaction.sampled = random.random() < rate
            print("hi there, set this bou")
            result = func(*args, **kwargs)
            if isawaitable(result):
                result = await result
            return result

        return wrapper

    return outer_wrapper



import typing as T
import json
from fastapi.encoders import jsonable_encoder
from graphql import FormattedExecutionResult
from strawberry.extensions import SchemaExtension
from strawberry.types import ExecutionContext
from app.dbs import db
from app.api.context import Context
from app import logs
from quilt import utils

logger = logs.create_logger(__name__)


def clean_result_obj(
    execution_context: ExecutionContext,
) -> dict[str, T.Any] | FormattedExecutionResult:
    if not execution_context.result:
        return {}
    try:
        res = execution_context.result.formatted
        if type(res) is not dict:
            res = {"res": str(res)}  # type: ignore
        return res
    except AttributeError as e:
        logger.error(f"{e=}, {execution_context.result=}")
    return {"res": str(execution_context.result)}


async def save_call(execution_context: ExecutionContext) -> None:
    context: Context = execution_context.context
    calls_config = context.config.calls
    if calls_config.ignore_all:
        logger.debug("ignoring call")
        return None
    if calls_config.ignore_variables:
        logger.debug("ignoring variables")
        variables = None
    else:
        variables = json.dumps(jsonable_encoder(execution_context.variables))
    request = context.request
    response = context.response
    insert = db.CallInsert(
        result=json.dumps(clean_result_obj(execution_context)),
        request_method=request.method,
        request_headers=json.dumps(jsonable_encoder(dict(request.headers))),
        request_url=str(request.url),
        request_id=execution_context.context.request_id,
        response_status_code=response.status_code,
        operation_name=execution_context.operation_name,
        query_str=execution_context.query,
        variables=variables,
        sentry_transaction_id=None,  # FUTURE add
    )

    if auth_id := context.auth.auth_id:
        insert.user = db.UserResolver().filter_by(auth_id=auth_id)
        if admin_auth_id := context.auth.admin_auth_id:
            insert.admin = db.UserResolver().filter_by(auth_id=admin_auth_id)

    call = await db.CallResolver().insert_one(insert=insert)
    logger.debug(f"INSERTED CALL, {call.id=}!")


class CallsExtension(SchemaExtension):
    async def on_request_end(self) -> None:
        if not utils.is_mutation(self.execution_context):
            return None
        self.execution_context.context.background_tasks.add_task(
            save_call, execution_context=self.execution_context
        )



import typing as T
import os
import sys
import re
from retry_async import retry
from . import schemas as S

from cloudinary.exceptions import Error as CloudinaryError
import cloudinary
import cloudinary.uploader
import cloudinary.api

cloudinary.config(
    cloud_name=os.environ["CLOUDINARY_CLOUD_NAME"],
    api_key=os.environ["CLOUDINARY_API_KEY"],
    api_secret=os.environ["CLOUDINARY_SECRET_KEY"],
)


def get_public_id_from_url(url: str) -> str | None:
    patt = "v\d*\/(.*)\."
    ids = re.findall(pattern=patt, string=url)
    if not ids or not len(ids):
        return None
    return ids[0]


def get_image(image_id: str) -> S.CloudinaryImage:
    resource = cloudinary.api.resource(image_id, phash=True)
    resource["url"] = resource.get("secure_url")
    return S.CloudinaryImage(**resource)


@retry(CloudinaryError, tries=5, delay=1, is_async=False)
def upload_image(url_or_bytes: T.Union[bytes, str]) -> S.CloudinaryImage:
    size = sys.getsizeof(url_or_bytes)
    print("size of url or bytes", size)
    res = cloudinary.uploader.upload(
        url_or_bytes, phash=True, folder=os.environ["CLOUDINARY_FOLDER_NAME"]
    )
    return S.CloudinaryImage(**res)



import typing as T
import datetime
from enum import Enum
from pydantic import BaseModel, Field, root_validator
import strawberry
import edge_orm
from edge_orm import UNSET, UnsetType
import sentry_sdk
from dataclasses import dataclass, asdict


def to_lower_camel(string: str) -> str:
    return "".join(
        word.capitalize() if i != 0 else word
        for i, word in enumerate(string.split("_"))
    )


class ParseFilterMixin(BaseModel):
    @root_validator(pre=True)
    def none_to_unset(cls, values: dict[str, T.Any]) -> dict[str, T.Any]:
        """For some reason on some nested fields, strawberry sends a None, not an UNSET. So much amke sure all
        Nones are not given, therefore defaulting to UNSET as set in the pydantic object.
        """
        return {k: v for k, v in values.items() if v is not None}

    @classmethod
    def from_filter(
        cls: T.Type["FilterPydanticType"], filter: dict[str, T.Any]
    ) -> "FilterPydanticType":
        """techincally also takes in strawberry input but can't find the type for this..."""
        if type(filter) is dict:
            d = filter
        else:
            d = asdict(filter)
            # d = {k: None if v is UNSET else v for k, v in d.items()}
        # TURN Nones into UNSETS -> idk why straw has this
        # d = {k: v for k, v in d.items() if v is not None} # this did not solve it
        node = cls.parse_obj(d)
        return node

    def add_filter_to_resolver(self, resolver: edge_orm.Resolver) -> None:
        from .logic import node_filter_to_str

        with sentry_sdk.start_span(
            op="Add Filter to Resolver", description=f"{resolver.__class__.__name__}"
        ):
            filter_str, variables = node_filter_to_str(f=self, root_field_name="")
            resolver.filter(filter_str, variables)

    def add_order_to_resolver(self, resolver: edge_orm.Resolver) -> None:
        from .logic import order_to_str

        with sentry_sdk.start_span(
            op="Add Order to Resolver", description=f"{resolver.__class__.__name__}"
        ):
            order_str = order_to_str(self)
            resolver.order_by(order_str)

    class Config:
        alias_generator = to_lower_camel
        allow_population_by_field_name = True
        arbitrary_types_allowed = True


FilterPydanticType = T.TypeVar("FilterPydanticType", bound=ParseFilterMixin)


class FilterString(ParseFilterMixin):
    in_: list[str] | UnsetType = Field(
        alias="in", default_factory=lambda: strawberry.UNSET
    )
    nin: list[str] | UnsetType = UNSET
    exists: bool | UnsetType = UNSET
    eq: str | UnsetType = UNSET
    neq: str | UnsetType = UNSET
    gte: str | UnsetType = UNSET
    gt: str | UnsetType = UNSET
    lte: str | UnsetType = UNSET
    lt: str | UnsetType = UNSET
    like: str | UnsetType = UNSET
    ilike: str | UnsetType = UNSET


@strawberry.experimental.pydantic.input(model=FilterString, name="FilterString")
class FilterStringStraw:
    in_: list[str] | None = strawberry.field(
        name="in", default_factory=lambda: strawberry.UNSET
    )
    nin: list[str] | None = UNSET
    exists: bool | None = UNSET
    eq: str | None = UNSET
    neq: str | None = UNSET
    gte: str | None = UNSET
    gt: str | None = UNSET
    lte: str | None = UNSET
    lt: str | None = UNSET
    like: str | None = UNSET
    ilike: str | None = UNSET


class FilterDateTime(FilterString):
    pass


FilterDateTimeStraw = strawberry.experimental.pydantic.input(
    model=FilterDateTime, name="FilterDateTime"
)(FilterStringStraw)


class FilterDate(ParseFilterMixin):
    in_: list[datetime.date] | UnsetType = Field(
        alias="in", default_factory=lambda: strawberry.UNSET
    )
    nin: list[datetime.date] | UnsetType = UNSET
    exists: bool | UnsetType = UNSET
    eq: datetime.date | UnsetType = UNSET
    neq: datetime.date | UnsetType = UNSET
    gte: datetime.date | UnsetType = UNSET
    gt: datetime.date | UnsetType = UNSET
    lte: datetime.date | UnsetType = UNSET
    lt: datetime.date | UnsetType = UNSET


@strawberry.experimental.pydantic.input(model=FilterDate, name="FilterDate")
class FilterDateStraw:
    in_: list[datetime.date] | None = strawberry.field(
        name="in", default_factory=lambda: strawberry.UNSET
    )
    nin: list[datetime.date] | None = UNSET
    exists: bool | None = UNSET
    eq: datetime.date | None = UNSET
    neq: datetime.date | None = UNSET
    gte: datetime.date | None = UNSET
    gt: datetime.date | None = UNSET
    lte: datetime.date | None = UNSET
    lt: datetime.date | None = UNSET


class FilterTime(ParseFilterMixin):
    in_: list[datetime.time] | UnsetType = Field(
        alias="in", default_factory=lambda: strawberry.UNSET
    )
    nin: list[datetime.time] | UnsetType = UNSET
    exists: bool | UnsetType = UNSET
    eq: datetime.time | UnsetType = UNSET
    neq: datetime.time | UnsetType = UNSET
    gte: datetime.time | UnsetType = UNSET
    gt: datetime.time | UnsetType = UNSET
    lte: datetime.time | UnsetType = UNSET
    lt: datetime.time | UnsetType = UNSET


@strawberry.experimental.pydantic.input(model=FilterTime, name="FilterTime")
class FilterTimeStraw:
    in_: list[datetime.time] | None = strawberry.field(
        name="in", default_factory=lambda: strawberry.UNSET
    )
    nin: list[datetime.time] | None = UNSET
    exists: bool | None = UNSET
    eq: datetime.time | None = UNSET
    neq: datetime.time | None = UNSET
    gte: datetime.time | None = UNSET
    gt: datetime.time | None = UNSET
    lte: datetime.time | None = UNSET
    lt: datetime.time | None = UNSET


class FilterJSON(FilterString):
    pass


FilterJSONStraw = strawberry.experimental.pydantic.input(
    model=FilterJSON, name="FilterJSON"
)(FilterStringStraw)


class FilterID(ParseFilterMixin):
    in_: list[str] | UnsetType = Field(
        alias="in", default_factory=lambda: strawberry.UNSET
    )
    nin: list[str] | UnsetType = UNSET
    exists: bool | UnsetType = UNSET
    eq: str | UnsetType = UNSET
    neq: str | UnsetType = UNSET


@strawberry.experimental.pydantic.input(model=FilterID, name="FilterID")
class FilterIDStraw:
    in_: list[str] | None = strawberry.field(
        name="in", default_factory=lambda: strawberry.UNSET
    )
    nin: list[str] | None = UNSET
    exists: bool | None = UNSET
    eq: str | None = UNSET
    neq: str | None = UNSET


class FilterInt(ParseFilterMixin):
    in_: list[int] | UnsetType = Field(
        alias="in", default_factory=lambda: strawberry.UNSET
    )
    nin: list[int] | UnsetType = UNSET
    exists: bool | UnsetType = UNSET
    eq: int | UnsetType = UNSET
    neq: int | UnsetType = UNSET
    gte: int | UnsetType = UNSET
    gt: int | UnsetType = UNSET
    lte: int | UnsetType = UNSET
    lt: int | UnsetType = UNSET


@strawberry.experimental.pydantic.input(model=FilterInt, name="FilterInt")
class FilterIntStraw:
    in_: list[int] | None = strawberry.field(
        name="in", default_factory=lambda: strawberry.UNSET
    )
    nin: list[int] | None = UNSET
    exists: bool | None = UNSET
    eq: int | None = UNSET
    neq: int | None = UNSET
    gte: int | None = UNSET
    gt: int | None = UNSET
    lte: int | None = UNSET
    lt: int | None = UNSET


class FilterInt16(FilterInt):
    pass


FilterInt16Straw = strawberry.experimental.pydantic.input(
    model=FilterInt16, name="FilterInt16"
)(FilterIntStraw)


class FilterInt32(FilterInt):
    pass


FilterInt32Straw = strawberry.experimental.pydantic.input(
    model=FilterInt32, name="FilterInt32"
)(FilterIntStraw)


class FilterInt64(FilterInt):
    pass


FilterInt64Straw = strawberry.experimental.pydantic.input(
    model=FilterInt64, name="FilterInt64"
)(FilterIntStraw)


class FilterFloat(ParseFilterMixin):
    in_: list[float] | UnsetType = Field(
        alias="in", default_factory=lambda: strawberry.UNSET
    )
    nin: list[float] | UnsetType = UNSET
    exists: bool | UnsetType = UNSET
    eq: float | UnsetType = UNSET
    neq: float | UnsetType = UNSET
    gte: float | UnsetType = UNSET
    gt: float | UnsetType = UNSET
    lte: float | UnsetType = UNSET
    lt: float | UnsetType = UNSET


@strawberry.experimental.pydantic.input(model=FilterFloat, name="FilterFloat")
class FilterFloatStraw:
    in_: list[float] | None = strawberry.field(
        name="in", default_factory=lambda: strawberry.UNSET
    )
    nin: list[float] | None = UNSET
    exists: bool | None = UNSET
    eq: float | None = UNSET
    neq: float | None = UNSET
    gte: float | None = UNSET
    gt: float | None = UNSET
    lte: float | None = UNSET
    lt: float | None = UNSET


class FilterFloat16(FilterFloat):
    pass


FilterFloat16Straw = strawberry.experimental.pydantic.input(
    model=FilterFloat16, name="FilterFloat16"
)(FilterFloatStraw)


class FilterFloat32(FilterFloat):
    pass


FilterFloat32Straw = strawberry.experimental.pydantic.input(
    model=FilterFloat32, name="FilterFloat32"
)(FilterFloatStraw)


class FilterFloat64(FilterFloat):
    pass


FilterFloat64Straw = strawberry.experimental.pydantic.input(
    model=FilterFloat64, name="FilterFloat64"
)(FilterFloatStraw)


class FilterBoolean(ParseFilterMixin):
    in_: list[bool] | UnsetType = Field(
        alias="in", default_factory=lambda: strawberry.UNSET
    )
    nin: list[bool] | UnsetType = UNSET
    exists: bool | UnsetType = UNSET
    eq: bool | UnsetType = UNSET
    neq: bool | UnsetType = UNSET


@strawberry.experimental.pydantic.input(model=FilterBoolean, name="FilterBoolean")
class FilterBooleanStraw:
    in_: list[bool] | None = strawberry.field(
        name="in", default_factory=lambda: strawberry.UNSET
    )
    nin: list[bool] | None = UNSET
    exists: bool | None = UNSET
    eq: bool | None = UNSET
    neq: bool | None = UNSET


class DirectionEnum(str, Enum):
    ASC = "ASC"
    DESC = "DESC"


strawberry.enum(DirectionEnum)


class NullsOrderingEnum(str, Enum):
    SMALLEST = "SMALLEST"
    BIGGEST = "BIGGEST"


strawberry.enum(NullsOrderingEnum)


class Ordering(ParseFilterMixin):
    dir: DirectionEnum
    nulls: NullsOrderingEnum | UnsetType = NullsOrderingEnum.SMALLEST


@strawberry.input(name="Ordering")
class OrderingStraw:
    dir: DirectionEnum
    nulls: NullsOrderingEnum | None = NullsOrderingEnum.SMALLEST


class FromDictMixin:
    pass
    """
    def to_pydantic(self) -> FilterPydanticType:
        return self.Config.pydantic_type.parse_obj(asdict(self))
    """


class AddToResolverMixin(FromDictMixin):
    def add_to_resolver(self, resolver: edge_orm.Resolver) -> None:
        from .logic import node_filter_to_str

        with sentry_sdk.start_span(
            op="Add Filter to Resolver", description=f"{resolver.__class__.__name__}"
        ):
            filter_str, variables = node_filter_to_str(f=self, root_field_name="")
            resolver.filter(filter_str, variables)


class AddToResolverMixinOrder(FromDictMixin):
    def add_to_resolver(self, resolver: edge_orm.Resolver) -> None:
        from .logic import order_to_str

        with sentry_sdk.start_span(
            op="Add Order to Resolver", description=f"{resolver.__class__.__name__}"
        ):
            resolver.order_by(order_to_str(self))



import asyncio
from quilt.vendors.GooglePlaces.main import Place, Address
from quilt.models.location import Location
from devtools import debug


async def get_place() -> Place:
    mad_green_place_id = "ChIJP5bmVaFZwokRyDdS3pPicdI"
    # invalid_place_id = mad_green_place_id[:-1] + "i"
    place = await Place.get(place_id=mad_green_place_id)
    assert place.place_id == mad_green_place_id
    return place


async def get_photos() -> None:
    place = await get_place()
    photo_urls = await place.get_photo_urls()
    print("photo_urls", photo_urls)
    assert len(photo_urls) > 1


async def main() -> None:
    # await get_photos()
    place = await get_place()
    debug(place)
    location = Location.from_place(place)
    debug(location)


if __name__ == "__main__":
    asyncio.run(main())



import strawberry


@strawberry.type
class NotFound:
    get_by_value: str | None = None
    message: str | None = "Not found."


@strawberry.type
class InvalidPermissions:
    get_by_value: str | None = None
    message: str | None = "You do not have permissions."



from . import logs, helpers
from .config import settings, Env, Platform
from .errors import TracesException, DisplayException
from .gql_utils.resolvers.startup import create_preloads
from .gql_utils.strawberry_utils import StrawMixin
from .gql_utils.resolvers.hydrate import rez_from_info
from .gql_utils.resolvers.models import preload, is_count, Func
from .types import NotFound, InvalidPermissions
from . import models, scalars

__all__ = [
    "logs",
    "helpers",
    "models",
    "scalars",
    "settings",
    "Env",
    "Platform",
    "TracesException",
    "DisplayException",
    "create_preloads",
    "StrawMixin",
    "rez_from_info",
    "preload",
    "is_count",
    "Func",
    "NotFound",
    "InvalidPermissions",
]



from .phone_number import PhoneNumber
from .image import Image, ImageStraw, ImageStrawInput, SmallImage, SmallImageStraw
from .input_datetime import InputDateTime, InputDateTimeDisplay
from . import location

__all__ = [
    "PhoneNumber",
    "Image",
    "ImageStraw",
    "ImageStrawInput",
    "InputDateTime",
    "location",
]



import strawberry
from . import LocationStraw, Location
from app import permissions


@strawberry.type
class LocationQuery:
    @strawberry.field(permission_classes=[permissions.Authed])
    async def location(self, place_id: str) -> LocationStraw:
        return LocationStraw.from_pydantic(
            await Location.from_place_id(place_id=place_id)
        )
